package App::Greple;

use strict;
use warnings;

=head1 NAME

greple - extensible grep with lexical expression and region handling

=head1 VERSION

Version 8.5702

=cut

our $VERSION = '8.5702';

=head1 AUTHOR

Kazumasa Utashiro

=head1 LICENSE AND COPYRIGHT

Copyright 1991-2022 Kazumasa Utashiro

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself.

=cut

1; # End of App::Greple
